#ifndef _MAUX_H_
#define _MAUX_H_

#include <stdint.h>
#include <iconv.h>
#include <sys/types.h>
#include <wctype.h>
#include <wchar.h>
#include <pthread.h>
#include <pwd.h>

#include <parsec/common.h>

#ifdef __cplusplus
extern "C" {
#endif
	extern char *maux_parse_word(const char *ptr, char *word, size_t size);
	extern int maux_parse_cfg(const char *cfg_name,
		int (*ent_parser) (void *cfg, const char *key,
		const char *value), void *cfg_struct, int *cnt);
	extern int maux_scnprintf(char * buf, size_t size, const char *fmt, ...);
	int maux_u8snprintf(char* str, size_t size, char *fmt,...);
/*
 * Function "strtou64" converts the string "data" to
 * number of type __u64 according to base "base".
 * Supplied base 2, 10 and 16 only.
 * "endptr" points to first non-convertable character:
 */
	extern uint64_t maux_strtou64(const char *data, char **endptr, int base);

/*
 * Function "u64tostr" converts number "data" to string "tostr",
 * according to base "base".
 * Supplied base 2, 10 and 16 only.
 * Returns pointer to result string (&tostr[0]).
 * In case of error NULL returned.
 */
	extern char *maux_u64tostr(char *tostr, uint64_t data, int base);
	extern uint16_t maux_le16tocpu(uint16_t var);
	extern uint32_t maux_le32tocpu(uint32_t var);
	extern uint64_t maux_le64tocpu(uint64_t var);
	extern uint16_t maux_cputole16(uint16_t var);
	extern uint32_t maux_cputole32(uint32_t var);
	extern uint64_t maux_cputole64(uint64_t var);

/* this is set to LE and get from LE only */
	extern unsigned char maux_get_w8(const void *p);
	extern unsigned short maux_get_w16(const void *p);
	extern unsigned int maux_get_w32(const void *p);
	extern unsigned long long maux_get_w64(const void *p);
	extern void maux_set_w8(void *p, unsigned char v);
	extern void maux_set_w16(void *p, unsigned short v);
	extern void maux_set_w32(void *p, unsigned long v);
	extern void maux_set_w64(void *p, unsigned long long v);

/* charset stuff */
	extern char *maux_encode(iconv_t hiconv, const char *s);
	extern wchar_t *maux_string_towc(const char *str);
	extern char *maux_string_fromwc(const wchar_t *str);

/* crc16 from linux kernel */
	extern uint16_t maux_crc16(uint8_t const *buffer, size_t len);

/* thread-safe file locks */
	struct __maux_flock_t {
		pthread_mutex_t m_mutex;
		pthread_cond_t m_cond;
		int m_rd; // RD locks counter
		int m_wr; // WR locks counter (may be 0 or 1 only)
	};

#define MAUX_FLOCK_INITIALIZER  {\
	.m_mutex = PTHREAD_MUTEX_INITIALIZER,\
	.m_cond = PTHREAD_COND_INITIALIZER,\
	.m_rd = 0,\
	.m_wr = 0\
}

#define INIT_MAUX_FLOCK(f) do { \
	pthread_mutex_init(&((f)->m_mutex), NULL); \
	pthread_cond_init(&((f)->m_cond), NULL); \
	(f)->m_rd = 0; \
	(f)->m_wr = 0; \
} while(0)

	typedef struct __maux_flock_t maux_flock_t;

	int maux_rdlock(maux_flock_t *mflock, int fd);
	int maux_wrlock(maux_flock_t *mflock, int fd);
	int maux_unlock(maux_flock_t *mflock, int fd);

/* wrappers for ugly getpwnam_r()-like functions */
	struct passwd *maux_getpwnam(const char *name);
	struct passwd *maux_getpwuid(uid_t uid);
	struct group *maux_getgrnam(const char *name);
	struct group *maux_getgrgid(gid_t gid);

/* helpers for local system files /etc/passwd, /etc/group */
	struct passwd *maux_fgetpwnam(const char *name);
	struct passwd *maux_fgetpwuid(uid_t uid);
	struct group *maux_fgetgrnam(const char *name);
	struct group *maux_fgetgrgid(gid_t gid);

/* thread safe basename and dirname */
	extern char *maux_basename (const char *filename);
	extern char *maux_dirname (const char *orig_path);
#ifdef __cplusplus
}
#endif

#endif
