/* $Id: maux_hash.h 194 2007-04-28 13:28:52Z pkun $ */

#ifndef __GHASH_H_INCLUDED
#define __GHASH_H_INCLUDED

/* generic hash */
#include <pthread.h>

#include <parsec/aux_list.h>

#define HASH_DEFAULT_SIZE 257

typedef struct _hash_table_item {
	struct	list_head hash_list;
} hash_table_item_t;

typedef struct _hash_list_item {
	struct 	list_head hash_list;
	struct	list_head list; /* just plain list */
	void	*key;
	size_t	key_size;
	void	*value;
} hash_list_item_t;

typedef unsigned long (*hash_fn_t)(const void *, size_t);
typedef void (*hash_free_fn)(void*);

typedef struct _hash_table {
	hash_table_item_t	*table; // table of list's (of table_size)
	unsigned long		table_size; // num of lists
	unsigned long		hash_size;  // num of items in hash
	hash_fn_t		hash_fn;
	unsigned long		last_hash_value;
	hash_list_item_t	*last_hash_access;
	struct list_head	list;
	pthread_mutex_t	hash_lock;
} hash_table_t;

extern hash_table_t *hash_init  (size_t size, hash_fn_t hash_fn);
extern void*         hash_find  (hash_table_t *h, const void *key, size_t key_size);
extern void*         hash_add    (hash_table_t *h, const void *key, size_t key_size, void *value);
extern void*         hash_del  (hash_table_t *h, const void *key, size_t key_size );
extern void          hash_done (hash_table_t *h);
extern void	     hash_unlock(hash_table_t *h);

#endif
